/*
 * Copyright (C) 2010 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.monophony;

import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;

import org.eiichiro.monophony.annotation.Endpoint;
import org.eiichiro.reverb.lang.ClassResolver;
import org.eiichiro.reverb.lang.UncheckedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@code WebInfClassesLoader} is a {@code Loader} implementation to load Web 
 * endpoint classes form 'WEB-INF/classes' directory.
 * If your Monophony application is running standalone (for unit testing), this 
 * class loads the classes from the classpath specified when your application 
 * was launched.
 * 
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
public class WebInfClassesLoader implements Loader {

	private Logger logger = LoggerFactory.getLogger(getClass());
	
	/**
	 * Loads Web endpoint classes from 'WEB-INF/classes' directory.
	 * 
	 * @see org.eiichiro.monophony.Loader#load()
	 */
	@Override
	public Set<Class<?>> load() {
		try {
			ClassResolver resolver = new ClassResolver();
			ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
			Enumeration<URL> resources = classLoader.getResources("");
			List<URL> urls = new ArrayList<URL>();
			
			for (int i = 0; resources.hasMoreElements(); i++) {
				URL url = resources.nextElement();
				logger.debug("Web endpoint search path #" + i + " [" + url + "]");
				urls.add(url);
			}
			
			resolver.setURLs(urls);
			Set<Class<?>> classes = resolver.resolveByAnnotation(Endpoint.class);
			logger.info("Web endpoints loaded");
			return classes;
		} catch (Exception e) {
			logger.error("Failed to load Web endpoint classes", e);
			throw new UncheckedException(e);
		}
	}

}
