/*
 * Copyright (C) 2010 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.monophony.json;

import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.lang.ClassUtils;
import org.eiichiro.monophony.AbstractParameter;
import org.eiichiro.monophony.Parameter;
import org.eiichiro.monophony.Types;
import org.eiichiro.monophony.WebContext;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

/**
 * {@code JSONBodyParameter} abstracts Web endpoint method parameter in HTTP 
 * request body sent as JSON (JavaScript Object Notation) format.
 * Web endpoint method parameter value is constructed from JSON with 
 * <a href='http://code.google.com/p/google-gson/'>Google Gson library</a>.
 * 
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
public class JSONBodyParameter extends AbstractParameter {

	private final JsonElement element;
	
	/**
	 * Constructs a new {@code JSONBodyParameter} with the specified Web context, 
	 * parameter type, parameter name and JSON element.
	 * 
	 * @param context Web context.
	 * @param type The parameter type.
	 * @param name The parameter name.
	 * @param element The parsed JSON element.
	 */
	protected JSONBodyParameter(WebContext context, Type type, String name,
			JsonElement element) {
		super(context, type, name, false);
		this.element = element;
		construct();
	}
	
	/**
	 * Constructs the value of Web endpoint parameter from HTTP request body 
	 * sent as JSON format.
	 * {@code JSONBodyParameter} supports the following parameter declaration 
	 * independently. Other than them are the same as {@code AbstractParameter}.
	 * <ol>
	 * <li>Named collection of user-defined object type</li>
	 * <li>Named user-defined object type</li>
	 * <li>No-named collection of core value type</li>
	 * <li>No-named collection of user-defined value type</li>
	 * <li>No-named collection of user-defined object type</li>
	 * </ol>
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected void construct() {
		if (name != null && !name.isEmpty()) {
			if (Types.isCollection(type)) {
				if (Types.isSupportedCollection(type)) {
					Class<?> elementType = Types.getElementType(type);
					
					if (!Types.isCoreValueType(elementType)
							&& !Types.isUserDefinedValueType(elementType)) {
						// Named collection of user-defined object type.
						Collection<Object> jsonElements = getValues(name);
						
						if (jsonElements == null) {
							logger.debug("Collection named [" + name + "] not found");
							return;
						}
						
						try {
							Class<?> implementationType = Types.getDefaultImplementationType(type);
							Collection<Object> collection = (Collection<Object>) implementationType.newInstance();
							
							try {
								for (Object jsonElement : jsonElements) {
									Object instance = elementType.newInstance();
									
									for (Field field : elementType.getDeclaredFields()) {
										Object object = new JSONBodyParameter(
												context, field.getGenericType(),
												field.getName(),
												(JsonElement) jsonElement).getValue();
										
										if (object != null) {
											field.setAccessible(true);
											field.set(instance, object);
										}
									}
									
									collection.add(instance);
								}
								
								value = (collection.isEmpty()) ? null : collection;
							} catch (Exception e) {
								logger.warn("Cannot instantiate [" + elementType
										+ "] (Collection element type of [" + type
										+ "])", e);
							}
							
						} catch (Exception e) {
							logger.debug("Cannot instantiate ["
									+ Types.getDefaultImplementationType(type)
									+ "] (Default implementation type of ["
									+ type + "])", e);
						}
						
						return;
					}
				}
				
			} else if (!Types.isCoreValueType(type)
					&& !Types.isUserDefinedValueType(type)) {
				// Named user-defined object type.
				Class<?> rawType = Types.getRawType(type);
				Object jsonElement = getValue(name);
				
				try {
					Object instance = rawType.newInstance();
					
					for (Field field : rawType.getDeclaredFields()) {
						Object object = new JSONBodyParameter(context, field
								.getGenericType(), field.getName(),
								(JsonElement) jsonElement).getValue();
						
						if (object != null) {
							field.setAccessible(true);
							field.set(instance, object);
						}
					}
					
					value = instance;
				} catch (Exception e) {
					logger.warn("Cannot instantiate [" + type + "]", e);
				}
				
				return;
			}
			
		} else {
			if (Types.isCollection(type)) {
				if (Types.isSupportedCollection(type)) {
					Collection<Object> values = getValues(name);
					
					if (values == null) {
						logger.debug("Posted JSON element is not a JSON array");
						return;
					}
					
					try {
						Class<?> implementationType = Types.getDefaultImplementationType(type);
						Collection<Object> collection = (Collection<Object>) implementationType.newInstance();
						Class<?> elementType = Types.getElementType(type);
						boolean coreValueType = Types.isCoreValueType(elementType);
						
						try {
							if (coreValueType || Types.isUserDefinedValueType(elementType)) {
								// No-named collection of core value type.
								// No-named collection of user-defined value type.
								for (Object object : values) {
									Object convert = (coreValueType) ? convert(object, elementType)
											: convertUserDefinedValueType(object, elementType);
									
									if (convert != null
											&& ClassUtils.primitiveToWrapper(elementType)
													.isAssignableFrom(convert.getClass())) {
										collection.add(convert);
									} else {
										logger.debug("Parameter [" + convert
												+ "] cannot be converted to ["
												+ elementType + "]");
									}
								}
								
							} else {
								// No-named collection of user-defined object type.
								for (Object jsonElement : values) {
									Object instance = elementType.newInstance();
									
									for (Field field : elementType.getDeclaredFields()) {
										Object object = new JSONBodyParameter(
												context, field.getGenericType(),
												field.getName(),
												(JsonElement) jsonElement).getValue();
										
										if (object != null) {
											field.setAccessible(true);
											field.set(instance, object);
										}
									}
									
									collection.add(instance);
								}
							}
							
							value = (collection.isEmpty()) ? null : collection;
						} catch (Exception e) {
							logger.warn("Cannot instantiate [" + elementType
									+ "] (Collection element type of [" + type
									+ "])", e);
						}
						
					} catch (Exception e) {
						logger.debug("Cannot instantiate ["
								+ Types.getDefaultImplementationType(type)
								+ "] (Default implementation type of ["
								+ type + "])", e);
					}
				}
				
				return;
			}
		}
		
		super.construct();
	}
	
	/**
	 * Converts the specified object to the specified type.
	 * The object to be converted must be a {@code JsonPrimitive}, or this 
	 * method return <code>null</code>.
	 * 
	 * @param object The object to be converted
	 * @param type The type to which the specified object be converted.
	 * @return The converted object.
	 */
	@Override
	protected Object convert(Object object, Class<?> type) {
		if (object instanceof JsonPrimitive) {
			JsonPrimitive primitive = (JsonPrimitive) object;
			return super.convert(primitive.getAsString(), type);
		} else {
			logger.warn("Parameter [" + object + "] cannot be converted to ["
					+ type + "]; Converted 'object' must be a ["
					+ JsonPrimitive.class + "]");
			return null;
		}
	}
	
	/**
	 * Converts the specified object to the specified user-defined value type.
	 * The object to be converted must be a {@code JsonPrimitive}, or this 
	 * method return <code>null</code>.
	 * 
	 * @param object The object to be converted
	 * @param type The user-defined value type to which the specified object be 
	 * converted.
	 * @return The converted object.
	 */
	@Override
	protected Object convertUserDefinedValueType(Object object, Class<?> type) {
		if (object instanceof JsonPrimitive) {
			JsonPrimitive primitive = (JsonPrimitive) object;
			return super.convertUserDefinedValueType(primitive.getAsString(), type);
		} else {
			logger.warn("Parameter [" + object + "] cannot be converted to ["
					+ type + "]; Converted 'object' must be a ["
					+ JsonPrimitive.class + "]");
			return null;
		}
	}
	
	// TODO: Support to specify the path of JSON objects by dotted name. 
	// For example, if 'child.name' is specified, this method returns 'name' 
	// field in 'child' JSON object.
	/**
	 * Returns the Web endpoint method parameter value corresponding to the 
	 * specified name from JSON content in HTTP request body.
	 * 
	 * @param name The parameter name.
	 * @return The Web endpoint method parameter value corresponding to the 
	 * specified name as {@code JsonElement}.
	 */
	@Override
	protected Object getValue(String name) {
		if (element instanceof JsonObject) {
			JsonObject jsonObject = (JsonObject) element;
			return jsonObject.get(name);
		} else {
			return null;
		}
	}

	// TODO: Support to specify the path of JSON objects by dotted name.
	/**
	 * Returns the Web endpoint method parameter value corresponding to the 
	 * specified name as a collection from JSON content in HTTP request body.
	 * If the posted element is JSON array, the specified parameter name may be 
	 * <code>null</code>.
	 * 
	 * @param name The parameter name.
	 * @return The Web endpoint method parameter value corresponding to the 
	 * specified name as collection of {@code JsonElement}.
	 * @since 0.2.0 The parameter name may be <code>null</code>, when the posted 
	 * element is JSON array.
	 */
	@Override
	protected Collection<Object> getValues(String name) {
		if (element instanceof JsonObject) {
			JsonObject jsonObject = (JsonObject) element;
			JsonElement jsonElement = jsonObject.get(name);
			
			if (jsonElement != null && jsonElement.isJsonArray()) {
				Collection<Object> collection = new ArrayList<Object>();
				Iterator<JsonElement> iterator = jsonElement.getAsJsonArray().iterator();
				
				while (iterator.hasNext()) {
					collection.add(iterator.next());
				}
				
				return (collection.isEmpty()) ? null : collection;
			} else {
				logger.warn("Element [" + name + "] is not a JSON array");
				return null;
			}
			
		} else if ((name == null || name.isEmpty()) && element instanceof JsonArray) {
			JsonArray array = (JsonArray) element;
			Collection<Object> collection = new ArrayList<Object>();
			Iterator<JsonElement> iterator = array.iterator();
			
			while (iterator.hasNext()) {
				collection.add(iterator.next());
			}
			
			return (collection.isEmpty()) ? null : collection;
		} else {
			return null;
		}
	}

	/**
	 * Returns a new {@code JSONBodyParameter} with the specified Web context, 
	 * parameter type and parameter name.
	 * 
	 * @param context Web context.
	 * @param type The parameter type.
	 * @param name The parameter name.
	 */
	@Override
	protected Parameter newParameter(WebContext context, Type type, String name) {
		return new JSONBodyParameter(context, type, name, element);
	}

}
