/*
 * Copyright (C) 2010 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.monophony.json;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.eiichiro.monophony.GenericResponse;
import org.eiichiro.monophony.MediaType;
import org.eiichiro.monophony.Result;
import org.eiichiro.monophony.WebContext;
import org.eiichiro.reverb.lang.UncheckedException;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

/**
 * {@code JSONResponse} is a JSON-based implementation of {@code Response}.
 * If the generated/negotiated MIME media type is 'application/json', Monophony 
 * uses this implementation.
 * Web endpoint method invocation result is deserialized to JSON with 
 * <a href='http://code.google.com/p/google-gson/'>Google Gson library</a>. 
 * UTF-8 is used for the character encoding (MIME charset) at any time.
 * 
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
public class JSONResponse extends GenericResponse {

	/**
	 * Constructs a new {@code JSONResponse} with the specified Web endpoint 
	 * method invocation result.
	 * 
	 * @param result Web endpoint invocation result.
	 */
	public JSONResponse(Result result) {
		super(result);
		mediaType = MediaType.APPLICATION_JSON;
	}
	
	/**
	 * Sends the Web endpoint invocation result to the client as the JSON-formed 
	 * HTTP response.
	 */
	@Override
	public void out(WebContext context) {
		Object value = result.getValue();
		
		if (value == null) {
			return;
		}
		
		if (value instanceof String) {
			super.out(context);
			return;
		}
		
		HttpServletResponse response = context.getResponse();
		response.setHeader("Content-Type", mediaType);
		
		if (status > 0) {
			response.setStatus(status);
		}
		
		// XXX: Any options?
		Gson gson = new GsonBuilder().create();
		
		try {
			response.setCharacterEncoding("UTF-8");
			response.getWriter().write(gson.toJson(value));
		} catch (IOException e) {
			throw new UncheckedException(e);
		}
	}

}
