/*
 * Copyright (C) 2010 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.monophony.xml;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.eiichiro.monophony.GenericResponse;
import org.eiichiro.monophony.MediaType;
import org.eiichiro.monophony.Result;
import org.eiichiro.monophony.WebContext;
import org.eiichiro.reverb.lang.UncheckedException;

/**
 * {@code XMLResponse} is a XML-based implementation of {@code Response}.
 * If the requested MIME media type is 'application/xml', default settings of 
 * Monophony uses this implementation.
 * Web endpoint method invocation result is deserialized to XML with JAXB (Java 
 * Architecture for XML Binding). UTF-8 is used for the character encoding 
 * (MIME charset) at any time.
 * 
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
public class XMLResponse extends GenericResponse {

	/**
	 * Constructs a new {@code XMLResponse} with the specified Web endpoint 
	 * method invocation result.
	 * @param result Web endpoint method invocation result.
	 */
	public XMLResponse(Result result) {
		super(result);
		mediaType = MediaType.APPLICATION_XML;
	}
	
	/**
	 * Sends the Web endpoint invocation result to the client as the XML-formed 
	 * HTTP response.
	 */
	@Override
	public void out(WebContext context) {
		Object value = result.getValue();
		
		if (value == null) {
			return;
		}
		
		if (value instanceof String) {
			super.out(context);
			return;
		}
		
		HttpServletResponse response = context.getResponse();
		response.setHeader("Content-Type", mediaType);
		
		if (status > 0) {
			response.setStatus(status);
		}
		
		try {
			response.setCharacterEncoding("UTF-8");
			Marshaller marshaller = JAXBContext.newInstance(value.getClass()).createMarshaller();
			marshaller.marshal(value, response.getWriter());
		} catch (Exception e) {
			throw new UncheckedException(e);
		}
	}

}
